@@ -1,44 +1,61 @@
 #!/usr/bin/env python
-# $Id: SetupTools.py,v 1.9 2007/11/03 19:44:38 tavis_rudd Exp $
-"""Some tools for extending and working with distutils
-
-CREDITS: This module borrows code and ideas from M.A. Lemburg's excellent setup
-tools for the mxBase package.
-
-"""
-
-__author__ = "Tavis Rudd <tavis@damnsimple.com>"
-__version__ = "$Revision: 1.9 $"[11:-2]
-
 import os
 from os import listdir
 import os.path
 from os.path import exists, isdir, isfile, join, splitext
+import sys
 import types
 import glob
 import string
 import traceback
 
 from distutils.core import setup
-if 'CHEETAH_USE_SETUPTOOLS' in os.environ:
-    # @@TR: Please note that this is for testing purposes only!  PEAK setuptools
-    # is not required or recommended for installing Cheetah.  Downstream
-    # package managers (linux distros, etc.) should *not* enable this.
+if not os.getenv('CHEETAH_INSTALL_WITHOUT_SETUPTOOLS'):
     try:
-        # use http://peak.telecommunity.com/DevCenter/setuptools if it's installed
-        # requires Py >=2.3
         from setuptools import setup
     except ImportError:   
         from distutils.core import setup
 
 from distutils.core import Command
+from distutils.command.build_ext import build_ext
 from distutils.command.install_data import install_data
+from distutils.errors import CCompilerError, DistutilsExecError, \
+    DistutilsPlatformError
 
 #imports from Cheetah ...
-from src.FileUtils import findFiles
+from cheetah.FileUtils import findFiles
+
+if sys.platform == 'win32' and sys.version_info > (2, 6):
+   # 2.6's distutils.msvc9compiler can raise an IOError when failing to
+   # find the compiler
+   ext_errors = (CCompilerError, DistutilsExecError, DistutilsPlatformError,
+                 IOError)
+else:
+   ext_errors = (CCompilerError, DistutilsExecError, DistutilsPlatformError)
 
 ##################################################
 ## CLASSES ##
+
+class BuildFailed(Exception):
+    pass
+
+class mod_build_ext(build_ext):
+    """A modified version of the distutils build_ext command that raises an
+    exception when building of the extension fails.
+    """
+
+    def run(self):
+        try:
+            build_ext.run(self)
+        except DistutilsPlatformError, x:
+            raise BuildFailed(x)
+
+    def build_extension(self, ext):
+        try:
+            build_ext.build_extension(self, ext)
+        except ext_errors, x:
+            raise BuildFailed(x)
+
    
 class mod_install_data(install_data):
     """A modified version of the disutils install_data command that allows data
@@ -60,8 +77,8 @@ class mod_install_data(install_data):
         data_files = self.get_inputs()
         
         for entry in data_files:
-            if type(entry) != types.StringType:
-                raise ValueError, 'The entries in "data_files" must be strings'
+            if not isinstance(entry, basestring):
+                raise ValueError('The entries in "data_files" must be strings')
             
             entry = string.join(string.split(entry, '/'), os.sep)
             # entry is a filename or glob pattern
@@ -117,25 +134,37 @@ def run_setup(configurations):
     """
     # Build parameter dictionary
     kws = {}
+    newkws = {}
     for configuration in configurations:
         kws.update(vars(configuration))
     for name, value in kws.items():
-        if name[:1] == '_' or \
-           type(value) not in (types.StringType,
-                               types.ListType,
-                               types.TupleType,
-                               types.DictType,
-                               types.IntType,
-                               ):
-            del kws[name]
+        if name[:1] == '_':
+            continue
+        if not isinstance(value, (basestring, list, tuple, dict, int)):
+            continue
+        newkws[name] = value
+    kws = newkws
 
     # Add setup extensions
     cmdclasses = {
+        'build_ext': mod_build_ext,
         'install_data': mod_install_data,
         }
 
     kws['cmdclass'] = cmdclasses
 
     # Invoke distutils setup
-    apply(setup, (), kws)
+    try:
+        setup(**kws)
+    except BuildFailed, x:
+        print("One or more C extensions failed to build.")
+        print("Details: %s" % x)
+        print("Retrying without C extensions enabled.")
+
+        del kws['ext_modules']
+        setup(**kws)
+
+        print("One or more C extensions failed to build.")
+        print("Performance enhancements will not be available.")
+        print("Pure Python installation succeeded.")
 
